# Generated from pddl.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .pddlParser import pddlParser
else:
    from pddlParser import pddlParser

# This class defines a complete listener for a parse tree produced by pddlParser.
class pddlListener(ParseTreeListener):

    # Enter a parse tree produced by pddlParser#pddlDoc.
    def enterPddlDoc(self, ctx:pddlParser.PddlDocContext):
        pass

    # Exit a parse tree produced by pddlParser#pddlDoc.
    def exitPddlDoc(self, ctx:pddlParser.PddlDocContext):
        pass


    # Enter a parse tree produced by pddlParser#domain.
    def enterDomain(self, ctx:pddlParser.DomainContext):
        pass

    # Exit a parse tree produced by pddlParser#domain.
    def exitDomain(self, ctx:pddlParser.DomainContext):
        pass


    # Enter a parse tree produced by pddlParser#domainName.
    def enterDomainName(self, ctx:pddlParser.DomainNameContext):
        pass

    # Exit a parse tree produced by pddlParser#domainName.
    def exitDomainName(self, ctx:pddlParser.DomainNameContext):
        pass


    # Enter a parse tree produced by pddlParser#requireDef.
    def enterRequireDef(self, ctx:pddlParser.RequireDefContext):
        pass

    # Exit a parse tree produced by pddlParser#requireDef.
    def exitRequireDef(self, ctx:pddlParser.RequireDefContext):
        pass


    # Enter a parse tree produced by pddlParser#typesDef.
    def enterTypesDef(self, ctx:pddlParser.TypesDefContext):
        pass

    # Exit a parse tree produced by pddlParser#typesDef.
    def exitTypesDef(self, ctx:pddlParser.TypesDefContext):
        pass


    # Enter a parse tree produced by pddlParser#typedNameList.
    def enterTypedNameList(self, ctx:pddlParser.TypedNameListContext):
        pass

    # Exit a parse tree produced by pddlParser#typedNameList.
    def exitTypedNameList(self, ctx:pddlParser.TypedNameListContext):
        pass


    # Enter a parse tree produced by pddlParser#singleTypeNameList.
    def enterSingleTypeNameList(self, ctx:pddlParser.SingleTypeNameListContext):
        pass

    # Exit a parse tree produced by pddlParser#singleTypeNameList.
    def exitSingleTypeNameList(self, ctx:pddlParser.SingleTypeNameListContext):
        pass


    # Enter a parse tree produced by pddlParser#r_type.
    def enterR_type(self, ctx:pddlParser.R_typeContext):
        pass

    # Exit a parse tree produced by pddlParser#r_type.
    def exitR_type(self, ctx:pddlParser.R_typeContext):
        pass


    # Enter a parse tree produced by pddlParser#primType.
    def enterPrimType(self, ctx:pddlParser.PrimTypeContext):
        pass

    # Exit a parse tree produced by pddlParser#primType.
    def exitPrimType(self, ctx:pddlParser.PrimTypeContext):
        pass


    # Enter a parse tree produced by pddlParser#functionsDef.
    def enterFunctionsDef(self, ctx:pddlParser.FunctionsDefContext):
        pass

    # Exit a parse tree produced by pddlParser#functionsDef.
    def exitFunctionsDef(self, ctx:pddlParser.FunctionsDefContext):
        pass


    # Enter a parse tree produced by pddlParser#functionList.
    def enterFunctionList(self, ctx:pddlParser.FunctionListContext):
        pass

    # Exit a parse tree produced by pddlParser#functionList.
    def exitFunctionList(self, ctx:pddlParser.FunctionListContext):
        pass


    # Enter a parse tree produced by pddlParser#atomicFunctionSkeleton.
    def enterAtomicFunctionSkeleton(self, ctx:pddlParser.AtomicFunctionSkeletonContext):
        pass

    # Exit a parse tree produced by pddlParser#atomicFunctionSkeleton.
    def exitAtomicFunctionSkeleton(self, ctx:pddlParser.AtomicFunctionSkeletonContext):
        pass


    # Enter a parse tree produced by pddlParser#functionSymbol.
    def enterFunctionSymbol(self, ctx:pddlParser.FunctionSymbolContext):
        pass

    # Exit a parse tree produced by pddlParser#functionSymbol.
    def exitFunctionSymbol(self, ctx:pddlParser.FunctionSymbolContext):
        pass


    # Enter a parse tree produced by pddlParser#functionType.
    def enterFunctionType(self, ctx:pddlParser.FunctionTypeContext):
        pass

    # Exit a parse tree produced by pddlParser#functionType.
    def exitFunctionType(self, ctx:pddlParser.FunctionTypeContext):
        pass


    # Enter a parse tree produced by pddlParser#constantsDef.
    def enterConstantsDef(self, ctx:pddlParser.ConstantsDefContext):
        pass

    # Exit a parse tree produced by pddlParser#constantsDef.
    def exitConstantsDef(self, ctx:pddlParser.ConstantsDefContext):
        pass


    # Enter a parse tree produced by pddlParser#predicatesDef.
    def enterPredicatesDef(self, ctx:pddlParser.PredicatesDefContext):
        pass

    # Exit a parse tree produced by pddlParser#predicatesDef.
    def exitPredicatesDef(self, ctx:pddlParser.PredicatesDefContext):
        pass


    # Enter a parse tree produced by pddlParser#atomicFormulaSkeleton.
    def enterAtomicFormulaSkeleton(self, ctx:pddlParser.AtomicFormulaSkeletonContext):
        pass

    # Exit a parse tree produced by pddlParser#atomicFormulaSkeleton.
    def exitAtomicFormulaSkeleton(self, ctx:pddlParser.AtomicFormulaSkeletonContext):
        pass


    # Enter a parse tree produced by pddlParser#predicate.
    def enterPredicate(self, ctx:pddlParser.PredicateContext):
        pass

    # Exit a parse tree produced by pddlParser#predicate.
    def exitPredicate(self, ctx:pddlParser.PredicateContext):
        pass


    # Enter a parse tree produced by pddlParser#typedVariableList.
    def enterTypedVariableList(self, ctx:pddlParser.TypedVariableListContext):
        pass

    # Exit a parse tree produced by pddlParser#typedVariableList.
    def exitTypedVariableList(self, ctx:pddlParser.TypedVariableListContext):
        pass


    # Enter a parse tree produced by pddlParser#singleTypeVarList.
    def enterSingleTypeVarList(self, ctx:pddlParser.SingleTypeVarListContext):
        pass

    # Exit a parse tree produced by pddlParser#singleTypeVarList.
    def exitSingleTypeVarList(self, ctx:pddlParser.SingleTypeVarListContext):
        pass


    # Enter a parse tree produced by pddlParser#constraints.
    def enterConstraints(self, ctx:pddlParser.ConstraintsContext):
        pass

    # Exit a parse tree produced by pddlParser#constraints.
    def exitConstraints(self, ctx:pddlParser.ConstraintsContext):
        pass


    # Enter a parse tree produced by pddlParser#structureDef.
    def enterStructureDef(self, ctx:pddlParser.StructureDefContext):
        pass

    # Exit a parse tree produced by pddlParser#structureDef.
    def exitStructureDef(self, ctx:pddlParser.StructureDefContext):
        pass


    # Enter a parse tree produced by pddlParser#actionDef.
    def enterActionDef(self, ctx:pddlParser.ActionDefContext):
        pass

    # Exit a parse tree produced by pddlParser#actionDef.
    def exitActionDef(self, ctx:pddlParser.ActionDefContext):
        pass


    # Enter a parse tree produced by pddlParser#actionSymbol.
    def enterActionSymbol(self, ctx:pddlParser.ActionSymbolContext):
        pass

    # Exit a parse tree produced by pddlParser#actionSymbol.
    def exitActionSymbol(self, ctx:pddlParser.ActionSymbolContext):
        pass


    # Enter a parse tree produced by pddlParser#actionDefBody.
    def enterActionDefBody(self, ctx:pddlParser.ActionDefBodyContext):
        pass

    # Exit a parse tree produced by pddlParser#actionDefBody.
    def exitActionDefBody(self, ctx:pddlParser.ActionDefBodyContext):
        pass


    # Enter a parse tree produced by pddlParser#precondition.
    def enterPrecondition(self, ctx:pddlParser.PreconditionContext):
        pass

    # Exit a parse tree produced by pddlParser#precondition.
    def exitPrecondition(self, ctx:pddlParser.PreconditionContext):
        pass


    # Enter a parse tree produced by pddlParser#goalDesc.
    def enterGoalDesc(self, ctx:pddlParser.GoalDescContext):
        pass

    # Exit a parse tree produced by pddlParser#goalDesc.
    def exitGoalDesc(self, ctx:pddlParser.GoalDescContext):
        pass


    # Enter a parse tree produced by pddlParser#fComp.
    def enterFComp(self, ctx:pddlParser.FCompContext):
        pass

    # Exit a parse tree produced by pddlParser#fComp.
    def exitFComp(self, ctx:pddlParser.FCompContext):
        pass


    # Enter a parse tree produced by pddlParser#atomicTermFormula.
    def enterAtomicTermFormula(self, ctx:pddlParser.AtomicTermFormulaContext):
        pass

    # Exit a parse tree produced by pddlParser#atomicTermFormula.
    def exitAtomicTermFormula(self, ctx:pddlParser.AtomicTermFormulaContext):
        pass


    # Enter a parse tree produced by pddlParser#term.
    def enterTerm(self, ctx:pddlParser.TermContext):
        pass

    # Exit a parse tree produced by pddlParser#term.
    def exitTerm(self, ctx:pddlParser.TermContext):
        pass


    # Enter a parse tree produced by pddlParser#durativeActionDef.
    def enterDurativeActionDef(self, ctx:pddlParser.DurativeActionDefContext):
        pass

    # Exit a parse tree produced by pddlParser#durativeActionDef.
    def exitDurativeActionDef(self, ctx:pddlParser.DurativeActionDefContext):
        pass


    # Enter a parse tree produced by pddlParser#daDefBody.
    def enterDaDefBody(self, ctx:pddlParser.DaDefBodyContext):
        pass

    # Exit a parse tree produced by pddlParser#daDefBody.
    def exitDaDefBody(self, ctx:pddlParser.DaDefBodyContext):
        pass


    # Enter a parse tree produced by pddlParser#daGD.
    def enterDaGD(self, ctx:pddlParser.DaGDContext):
        pass

    # Exit a parse tree produced by pddlParser#daGD.
    def exitDaGD(self, ctx:pddlParser.DaGDContext):
        pass


    # Enter a parse tree produced by pddlParser#prefTimedGD.
    def enterPrefTimedGD(self, ctx:pddlParser.PrefTimedGDContext):
        pass

    # Exit a parse tree produced by pddlParser#prefTimedGD.
    def exitPrefTimedGD(self, ctx:pddlParser.PrefTimedGDContext):
        pass


    # Enter a parse tree produced by pddlParser#timedGD.
    def enterTimedGD(self, ctx:pddlParser.TimedGDContext):
        pass

    # Exit a parse tree produced by pddlParser#timedGD.
    def exitTimedGD(self, ctx:pddlParser.TimedGDContext):
        pass


    # Enter a parse tree produced by pddlParser#timeSpecifier.
    def enterTimeSpecifier(self, ctx:pddlParser.TimeSpecifierContext):
        pass

    # Exit a parse tree produced by pddlParser#timeSpecifier.
    def exitTimeSpecifier(self, ctx:pddlParser.TimeSpecifierContext):
        pass


    # Enter a parse tree produced by pddlParser#interval.
    def enterInterval(self, ctx:pddlParser.IntervalContext):
        pass

    # Exit a parse tree produced by pddlParser#interval.
    def exitInterval(self, ctx:pddlParser.IntervalContext):
        pass


    # Enter a parse tree produced by pddlParser#derivedDef.
    def enterDerivedDef(self, ctx:pddlParser.DerivedDefContext):
        pass

    # Exit a parse tree produced by pddlParser#derivedDef.
    def exitDerivedDef(self, ctx:pddlParser.DerivedDefContext):
        pass


    # Enter a parse tree produced by pddlParser#fExp.
    def enterFExp(self, ctx:pddlParser.FExpContext):
        pass

    # Exit a parse tree produced by pddlParser#fExp.
    def exitFExp(self, ctx:pddlParser.FExpContext):
        pass


    # Enter a parse tree produced by pddlParser#fExp2.
    def enterFExp2(self, ctx:pddlParser.FExp2Context):
        pass

    # Exit a parse tree produced by pddlParser#fExp2.
    def exitFExp2(self, ctx:pddlParser.FExp2Context):
        pass


    # Enter a parse tree produced by pddlParser#fHead.
    def enterFHead(self, ctx:pddlParser.FHeadContext):
        pass

    # Exit a parse tree produced by pddlParser#fHead.
    def exitFHead(self, ctx:pddlParser.FHeadContext):
        pass


    # Enter a parse tree produced by pddlParser#effect.
    def enterEffect(self, ctx:pddlParser.EffectContext):
        pass

    # Exit a parse tree produced by pddlParser#effect.
    def exitEffect(self, ctx:pddlParser.EffectContext):
        pass


    # Enter a parse tree produced by pddlParser#cEffect.
    def enterCEffect(self, ctx:pddlParser.CEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#cEffect.
    def exitCEffect(self, ctx:pddlParser.CEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#pEffect.
    def enterPEffect(self, ctx:pddlParser.PEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#pEffect.
    def exitPEffect(self, ctx:pddlParser.PEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#condEffect.
    def enterCondEffect(self, ctx:pddlParser.CondEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#condEffect.
    def exitCondEffect(self, ctx:pddlParser.CondEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#binaryOp.
    def enterBinaryOp(self, ctx:pddlParser.BinaryOpContext):
        pass

    # Exit a parse tree produced by pddlParser#binaryOp.
    def exitBinaryOp(self, ctx:pddlParser.BinaryOpContext):
        pass


    # Enter a parse tree produced by pddlParser#binaryComp.
    def enterBinaryComp(self, ctx:pddlParser.BinaryCompContext):
        pass

    # Exit a parse tree produced by pddlParser#binaryComp.
    def exitBinaryComp(self, ctx:pddlParser.BinaryCompContext):
        pass


    # Enter a parse tree produced by pddlParser#assignOp.
    def enterAssignOp(self, ctx:pddlParser.AssignOpContext):
        pass

    # Exit a parse tree produced by pddlParser#assignOp.
    def exitAssignOp(self, ctx:pddlParser.AssignOpContext):
        pass


    # Enter a parse tree produced by pddlParser#durationConstraint.
    def enterDurationConstraint(self, ctx:pddlParser.DurationConstraintContext):
        pass

    # Exit a parse tree produced by pddlParser#durationConstraint.
    def exitDurationConstraint(self, ctx:pddlParser.DurationConstraintContext):
        pass


    # Enter a parse tree produced by pddlParser#simpleDurationConstraint.
    def enterSimpleDurationConstraint(self, ctx:pddlParser.SimpleDurationConstraintContext):
        pass

    # Exit a parse tree produced by pddlParser#simpleDurationConstraint.
    def exitSimpleDurationConstraint(self, ctx:pddlParser.SimpleDurationConstraintContext):
        pass


    # Enter a parse tree produced by pddlParser#durOp.
    def enterDurOp(self, ctx:pddlParser.DurOpContext):
        pass

    # Exit a parse tree produced by pddlParser#durOp.
    def exitDurOp(self, ctx:pddlParser.DurOpContext):
        pass


    # Enter a parse tree produced by pddlParser#durValue.
    def enterDurValue(self, ctx:pddlParser.DurValueContext):
        pass

    # Exit a parse tree produced by pddlParser#durValue.
    def exitDurValue(self, ctx:pddlParser.DurValueContext):
        pass


    # Enter a parse tree produced by pddlParser#daEffect.
    def enterDaEffect(self, ctx:pddlParser.DaEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#daEffect.
    def exitDaEffect(self, ctx:pddlParser.DaEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#timedEffect.
    def enterTimedEffect(self, ctx:pddlParser.TimedEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#timedEffect.
    def exitTimedEffect(self, ctx:pddlParser.TimedEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#fAssignDA.
    def enterFAssignDA(self, ctx:pddlParser.FAssignDAContext):
        pass

    # Exit a parse tree produced by pddlParser#fAssignDA.
    def exitFAssignDA(self, ctx:pddlParser.FAssignDAContext):
        pass


    # Enter a parse tree produced by pddlParser#fExpDA.
    def enterFExpDA(self, ctx:pddlParser.FExpDAContext):
        pass

    # Exit a parse tree produced by pddlParser#fExpDA.
    def exitFExpDA(self, ctx:pddlParser.FExpDAContext):
        pass


    # Enter a parse tree produced by pddlParser#assignOpT.
    def enterAssignOpT(self, ctx:pddlParser.AssignOpTContext):
        pass

    # Exit a parse tree produced by pddlParser#assignOpT.
    def exitAssignOpT(self, ctx:pddlParser.AssignOpTContext):
        pass


    # Enter a parse tree produced by pddlParser#problem.
    def enterProblem(self, ctx:pddlParser.ProblemContext):
        pass

    # Exit a parse tree produced by pddlParser#problem.
    def exitProblem(self, ctx:pddlParser.ProblemContext):
        pass


    # Enter a parse tree produced by pddlParser#problemDecl.
    def enterProblemDecl(self, ctx:pddlParser.ProblemDeclContext):
        pass

    # Exit a parse tree produced by pddlParser#problemDecl.
    def exitProblemDecl(self, ctx:pddlParser.ProblemDeclContext):
        pass


    # Enter a parse tree produced by pddlParser#problemDomain.
    def enterProblemDomain(self, ctx:pddlParser.ProblemDomainContext):
        pass

    # Exit a parse tree produced by pddlParser#problemDomain.
    def exitProblemDomain(self, ctx:pddlParser.ProblemDomainContext):
        pass


    # Enter a parse tree produced by pddlParser#objectDecl.
    def enterObjectDecl(self, ctx:pddlParser.ObjectDeclContext):
        pass

    # Exit a parse tree produced by pddlParser#objectDecl.
    def exitObjectDecl(self, ctx:pddlParser.ObjectDeclContext):
        pass


    # Enter a parse tree produced by pddlParser#init.
    def enterInit(self, ctx:pddlParser.InitContext):
        pass

    # Exit a parse tree produced by pddlParser#init.
    def exitInit(self, ctx:pddlParser.InitContext):
        pass


    # Enter a parse tree produced by pddlParser#initEl.
    def enterInitEl(self, ctx:pddlParser.InitElContext):
        pass

    # Exit a parse tree produced by pddlParser#initEl.
    def exitInitEl(self, ctx:pddlParser.InitElContext):
        pass


    # Enter a parse tree produced by pddlParser#nameLiteral.
    def enterNameLiteral(self, ctx:pddlParser.NameLiteralContext):
        pass

    # Exit a parse tree produced by pddlParser#nameLiteral.
    def exitNameLiteral(self, ctx:pddlParser.NameLiteralContext):
        pass


    # Enter a parse tree produced by pddlParser#atomicNameFormula.
    def enterAtomicNameFormula(self, ctx:pddlParser.AtomicNameFormulaContext):
        pass

    # Exit a parse tree produced by pddlParser#atomicNameFormula.
    def exitAtomicNameFormula(self, ctx:pddlParser.AtomicNameFormulaContext):
        pass


    # Enter a parse tree produced by pddlParser#goal.
    def enterGoal(self, ctx:pddlParser.GoalContext):
        pass

    # Exit a parse tree produced by pddlParser#goal.
    def exitGoal(self, ctx:pddlParser.GoalContext):
        pass


    # Enter a parse tree produced by pddlParser#probConstraints.
    def enterProbConstraints(self, ctx:pddlParser.ProbConstraintsContext):
        pass

    # Exit a parse tree produced by pddlParser#probConstraints.
    def exitProbConstraints(self, ctx:pddlParser.ProbConstraintsContext):
        pass


    # Enter a parse tree produced by pddlParser#prefConGD.
    def enterPrefConGD(self, ctx:pddlParser.PrefConGDContext):
        pass

    # Exit a parse tree produced by pddlParser#prefConGD.
    def exitPrefConGD(self, ctx:pddlParser.PrefConGDContext):
        pass


    # Enter a parse tree produced by pddlParser#metricSpec.
    def enterMetricSpec(self, ctx:pddlParser.MetricSpecContext):
        pass

    # Exit a parse tree produced by pddlParser#metricSpec.
    def exitMetricSpec(self, ctx:pddlParser.MetricSpecContext):
        pass


    # Enter a parse tree produced by pddlParser#optimization.
    def enterOptimization(self, ctx:pddlParser.OptimizationContext):
        pass

    # Exit a parse tree produced by pddlParser#optimization.
    def exitOptimization(self, ctx:pddlParser.OptimizationContext):
        pass


    # Enter a parse tree produced by pddlParser#metricFExp.
    def enterMetricFExp(self, ctx:pddlParser.MetricFExpContext):
        pass

    # Exit a parse tree produced by pddlParser#metricFExp.
    def exitMetricFExp(self, ctx:pddlParser.MetricFExpContext):
        pass


    # Enter a parse tree produced by pddlParser#conGD.
    def enterConGD(self, ctx:pddlParser.ConGDContext):
        pass

    # Exit a parse tree produced by pddlParser#conGD.
    def exitConGD(self, ctx:pddlParser.ConGDContext):
        pass


    # Enter a parse tree produced by pddlParser#name.
    def enterName(self, ctx:pddlParser.NameContext):
        pass

    # Exit a parse tree produced by pddlParser#name.
    def exitName(self, ctx:pddlParser.NameContext):
        pass


